@@ -197,15 +197,7 @@ impl BundleSavefile {
   }
 }
 
-#[throws(fmt::Error)]
-fn fmt_hex(f: &mut Formatter, buf: &[u8]) {
-  for v in buf { write!(f, "{:02x}", v)?; }
-}
-
-impl Debug for Hash {
-  #[throws(fmt::Error)]
-  fn fmt(&self, f: &mut Formatter) { fmt_hex(f, &self.0)? }
-}
+format_by_fmt_hex!{Debug, for Hash, .0}
 impl Display for Hash {
   #[throws(fmt::Error)]
   fn fmt(&self, f: &mut Formatter) {
@@ -273,7 +265,7 @@ impl Id {
 
   pub fn path(&self, instance: &Unauthorised<InstanceGuard<'_>, InstanceName>,
           auth: Authorisation<Id>) -> String {
-    self.path_(&instance.by_ref(auth.therefore_ok()).name)
+    self.path_(&instance.by_ref(auth.so_promise()).name)
   }
 
   #[throws(IE)]
@@ -292,7 +284,7 @@ impl Id {
   #[throws(IE)]
   pub fn open(&self, instance: &Instance) -> Option<fs::File> {
     let name = &*instance.name;
-    let auth = Authorisation::authorised(name).bundles();
+    let auth = Authorisation::promise_for(name).bundles();
     self.open_by_name(name, auth)?
   }
 
@@ -304,7 +296,7 @@ impl Id {
 //---------- displaing/presenting/authorising ----------
 
 impl Authorisation<InstanceName> {
-  pub fn bundles(self) -> Authorisation<Id> { self.therefore_ok() }
+  pub fn bundles(self) -> Authorisation<Id> { self.so_promise() }
 }
 
 impl Display for State {
@@ -897,6 +889,43 @@ fn make_usvg(instance_name: &str, bundle_name: &str, za: &mut IndexedZip,
 
 //---------- specs ----------
 
+#[throws(anyhow::Error)]
+pub fn spec_macroexpand(
+  input: String,
+  report: &mut dyn FnMut(&'static str, &str) -> Result<(),AE>,
+) -> String {
+  if ! input.starts_with("{#") { return input }
+
+  let templates: Vec<(&str, Cow<str>)> = match input.rfind("\n{% endmacro") {
+    None => vec![ ("spec", input.into()) ],
+    Some(endm_base) => {
+      let endm_sol = endm_base + 1;
+      let endm_end = endm_sol + 1 +
+        input[endm_sol..].find('\n')
+        .ok_or_else(|| anyhow!("endmacro line not terminated"))?;
+      let mac_data = &input[0..endm_end];
+      let spec_data = 
+        r#"{% import "m" as m %}"#.to_string()
+        + &"\n".repeat(mac_data.matches('\n').count())
+        + &input[endm_end..];
+      vec![ ("m",    mac_data .into()),
+            ("spec", spec_data.into()) ]
+    },
+  };
+
+  for (nomfile, data) in &templates { report(nomfile, data)?; }
+
+  let mut tera = tera_standalone::Tera::default();
+  tera.add_raw_templates(templates).context("load")?;
+  let mut out: Vec<u8> = vec![];
+  tera.render_to("spec", &default(), &mut out).context("render")?;
+  let out = String::from_utf8(out).context("reparse as utf-8")?;
+
+  report("out", &out)?;
+
+  out
+}
+
 #[throws(MgmtError)]
 pub fn load_spec_to_read(ig: &Instance, spec_name: &str) -> String {
   #[throws(MgmtError)]
@@ -908,24 +937,31 @@ pub fn load_spec_to_read(ig: &Instance, spec_name: &str) -> String {
       ErrorKind::UnexpectedEof => ME::BadBundle(e.to_string()),
       _ => e_f(e),
     })?;
-    buf
+
+    spec_macroexpand(buf, &mut |_,_|Ok(()))
+      .map_err(|ae| ME::BadBundle(
+        format!("process spec as Tera template: {}", ae.d())
+      ))?
   }
 
   let spec_leaf = format!("{}.game.toml", spec_name);
 
   if let Some((id, index)) = ig.bundle_specs.get(&UniCase::from(spec_name)) {
-    let fpath = id.path_(&ig.name);
-    let f = File::open(&fpath)
-      .with_context(|| fpath.clone()).context("reopen bundle")
-      .map_err(IE::from)?;
     match id.kind {
       Kind::Zip => {
+
+        let fpath = id.path_(&ig.name);
+        let f = File::open(&fpath)
+          .with_context(|| fpath.clone()).context("reopen bundle")
+          .map_err(IE::from)?;
+
         let mut za = ZipArchive::new(BufReader::new(f)).map_err(
           |e| LE::BadBundle(format!("re-examine zipfile: {}", e)))?;
         let mut f = za.i(*index).map_err(
           |e| LE::BadBundle(format!("re-find zipfile member: {}", e)))?;
         return read_from_read(&mut f, &mut |e|{
           LE::BadBundle(format!("read zipfile member: {}", e))}.into())?;
+
       }
     }
   }
