@@ -642,3 +642,194 @@ impl<I,T> IndexVec<I,T> where I: index_vec::Idx {
     self.raw.get_or_extend_with(i.index(), f)
   }
 }
+
+#[derive(Debug)]
+pub struct AnyhowDisplay<'a>(pub &'a anyhow::Error);
+impl Display for AnyhowDisplay<'_> {
+  #[throws(fmt::Error)]
+  fn fmt(&self, f: &mut fmt::Formatter) {
+    let mut delim = "";
+    self.0.for_each(&mut |s|{
+      write!(f, "{}{}", delim, s)?;
+      delim = ": ";
+      Ok(())
+    })?;
+  }
+}
+
+#[ext(pub)]
+impl anyhow::Error {
+  fn for_each(&self, f: &mut dyn FnMut(&str) -> fmt::Result) -> fmt::Result {
+    let mut done = String::new();
+    for e in self.chain() {
+      let s = e.to_string();
+      if done.contains(&s) { continue }
+      f(&s)?;
+      done = s;
+    }
+    Ok(())
+  }
+
+  fn d(&self) -> AnyhowDisplay<'_> { AnyhowDisplay(self) }
+
+  fn end_process(self, estatus: u8) -> ! {
+    #[derive(Default,Debug)] struct Sol { any: bool, progname: String }
+    impl Sol {
+      fn nl(&mut self) {
+        if self.any { eprintln!("") };
+        self.any = false;
+      }
+      fn head(&mut self) {
+        if ! self.any { eprint!("{}: error", &self.progname); }
+        self.any = true
+      }
+    }
+    let mut sol: Sol = Sol { any: false, progname: program_name() };
+    self.for_each(&mut |s|{
+      let long = s.len() > 80;
+      if long && sol.any { sol.nl() }
+      sol.head();
+      eprint!(": {}", &s);
+      if long { sol.nl() }
+      Ok::<_,fmt::Error>(())
+    }).unwrap();
+    sol.nl();
+    assert!(estatus > 0);
+    exit(estatus.into());
+  }
+}
+
+#[throws(Either<io::Error, io::Error>)]
+pub fn io_copy_interactive<R,W>(read: &mut BufReader<R>, write: &mut W)
+where R: Read, W: Write {
+  loop {
+    let buf = read.fill_buf().map_err(Either::Left)?;
+    if buf.len() == 0 { break }
+
+    let did = (||{
+      let did = write.write(buf)?;
+      if did == 0 { throw!(ErrorKind::WriteZero) }
+      Ok::<_,io::Error>(did)
+    })().map_err(Either::Right)?;
+        
+    read.consume(did);
+    write.flush().map_err(Either::Right)?;
+  }
+}
+
+pub struct SigPipeWriter<W>(pub W);
+
+impl<W:Write> SigPipeWriter<W> {
+  fn handle_err(e: io::Error) -> io::Error {
+    if e.kind() != ErrorKind::BrokenPipe { return e }
+
+    match (||{
+      use nix::sys::signal::*;
+      use Signal::SIGPIPE;
+      unsafe {
+        sigaction(SIGPIPE, &SigAction::new(
+          SigHandler::SigDfl, SaFlags::empty(), SigSet::empty()))
+          .context("sigaction")?;
+      };
+      raise(SIGPIPE).context("raise")?;
+      Err::<Void,_>(anyhow!("continued after raise"))
+    })() {
+      Err(ae) => ae
+        .context("attempt to die with SIGPIPE failed")
+        .end_process(127),
+      Ok(v) => match v { },
+    }
+  }
+}
+
+impl<W:Write> Write for SigPipeWriter<W> {
+  fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
+    self.0.write(buf).map_err(Self::handle_err)
+  }
+  fn flush(&mut self)             -> io::Result<()>    {
+    self.0.flush()   .map_err(Self::handle_err)
+  }
+}
+
+pub type RawStdout = SigPipeWriter<io::Stdout>;
+impl RawStdout { pub fn new() -> Self { SigPipeWriter(io::stdout()) } }
+
+pub struct CookedStdout(pub BufWriter<SigPipeWriter<io::Stdout>>);
+impl CookedStdout {
+  pub fn new() -> Self { Self(BufWriter::new(RawStdout::new())) }
+  fn handle_err(e: io::Error) -> ! {
+    AE::from(e).context("write stdout").end_process(EXIT_DISASTER as _);
+  }
+  fn must_flush(&mut self) {
+    self.0.flush().unwrap_or_else(|e| Self::handle_err(e))
+  }
+}
+impl Write for CookedStdout {
+  #[throws(io::Error)]
+  fn write(&mut self, buf: &[u8]) -> usize {
+    let r = self.0.write(buf).unwrap_or_else(|e| Self::handle_err(e));
+    if buf.contains(&b'\n') { self.flush()? }
+    r
+  }
+  #[throws(io::Error)]
+  fn flush(&mut self) { self.must_flush() }
+}
+impl Drop for CookedStdout {
+  fn drop(&mut self) { self.must_flush() }
+}
+
+#[throws(fmt::Error)]
+pub fn fmt_hex(f: &mut Formatter, buf: &[u8]) {
+  for v in buf { write!(f, "{:02x}", v)?; }
+}
+
+#[throws(as Option)]
+#[must_use]
+pub fn parse_slice_hex(s: &str, buf: &mut [u8]) -> usize {
+  let l = s.len();
+  if l % 1 != 0 { throw!() }
+  let l = l/2;
+  if l > buf.len() { throw!() }
+
+  for (h, o) in izip!(
+    s.as_bytes().chunks(2),
+    buf.iter_mut(),
+  ) {
+    let h = str::from_utf8(h).ok()?;
+    *o = u8::from_str_radix(h,16).ok()?;
+  }
+
+  l
+}
+
+#[throws(as Option)]
+pub fn parse_fixed_hex<const N: usize>(s: &str) -> [u8; N] {
+  let mut buf = [0u8; N];
+  let l = parse_slice_hex(s, &mut buf)?;
+  if l != N { throw!() }
+  buf
+}
+
+#[macro_export]
+macro_rules! format_by_fmt_hex {
+  ($trait:ty, for $self:ty, . $($memb:tt)+) => {
+    impl $trait for $self {
+      #[throws(fmt::Error)]
+      fn fmt(&self, f: &mut Formatter) {
+        fmt_hex(f, &self . $($memb)+)?
+      }
+    }
+  }
+}
+
+#[test]
+fn test_parse_hex(){
+  assert_eq!( parse_fixed_hex(""),     Some([          ]) );
+  assert_eq!( parse_fixed_hex("41"  ), Some([b'A'      ]) );
+  assert_eq!( parse_fixed_hex("4165"), Some([b'A', b'e']) );
+  assert_eq!( parse_fixed_hex("4165"), Some([b'A', b'e']) );
+  assert_eq!( parse_fixed_hex("41"  ), None::<[_;0]>      );
+  assert_eq!( parse_fixed_hex("41"  ), None::<[_;2]>      );
+  assert_eq!( parse_fixed_hex("1"   ), None::<[_;1]>      );
+  assert_eq!( parse_fixed_hex("xy"  ), None::<[_;1]>      );
+}
